/*******************************************************************************
This is the main C code for the WiFi Sprinkler.  7/17/2010   Bob Colwell

This code assumes a Rabbit RCM5450W wifi module.
This code assumes a board rev of 8.0 or higher.

8/26/10 bug fix: zone-- for auto mode too, not just manual (fixed in lib)
8/31/10 bug fix: removed "a:" from SPRNKLR.txt path in boot_file_setup()
8/31/10 improvement: Ellen edited the zones html file for better clarity

*******************************************************************************/
#class auto
#define SET_RTC

/* Note that these #defines are seen by the compiler
 * before the #use reference to the libs
 */
#define CODE_REV 14
#define CODE_SUBREV 5
unsigned int code_rev, code_subrev;

unsigned int no_lcd_events, lcd_beyond_initial_startup;

int ledflag[17];   // 16 flags; zeroth is not used
int sprinkling;    // boolean, TRUE when any zone is sprinkling

/*
 * NETWORK CONFIGURATION
 */
#define TCPCONFIG 1
#define _PRIMARY_STATIC_IP  "192.168.2.55"
#define _PRIMARY_NETMASK    "255.255.255.0"
#define MY_GATEWAY          "192.168.2.1"
#define MY_NAMESERVER       "192.168.2.1"
/* REMEMBER TO HIDE NEXT LINE BEFORE PUBLISHING!! */
#define IFC_WIFI_SSID       "ColwellWirelessNetwork"
#define IFC_WIFI_MODE       IFPARAM_WIFI_INFRASTRUCTURE
#define IFC_WIFI_REGION     IFPARAM_WIFI_REGION_AMERICAS
#define IFC_WIFI_ENCRYPTION IFPARAM_WIFI_ENCR_WEP
/* REMEMBER TO HIDE NEXT LINE BEFORE PUBLISHING!! */
#define IFC_WIFI_WEP_KEY0_HEXSTR  "66544B0E01"

// If TIMEZONE not defined, default to "UTC" (0h).
#define TIMEZONE        (-8)	// Hours difference from UTC (whole hours only)

#ifdef TIMEZONE
	#define TZNAME		"PST"	// Standard name for above timezone
#else
	#define TIMEZONE	 0
	#define TZNAME		"UTC"
#endif

// Hostnames (or dotted quad IP addresses) of each time server to query.
char * const server_hosts[] =
{
   "time-b.nist.gov"
  ,"nist1.aol-ca.truetime.com"
  ,"0.pool.ntp.org"
  ,"1.pool.ntp.org"
  ,"2.pool.ntp.org"
  ,"pool.ntp.org"
};

#define NSERVERS (sizeof(server_hosts)/sizeof(server_hosts[0]))

/*
 * Define the number of HTTP servers and socket buffers.
 * With tcp_reserveport(), fewer HTTP servers are needed.
 */
#define HTTP_MAXSERVERS 2
#define MAX_TCP_SOCKET_BUFFERS 2

/********************************
 * End of configuration section *
 ********************************/

/*
 * Enable RabbitWeb HTTP enhancements & ZHTML scripting language.
 */
#define USE_RABBITWEB 1

#define REDIRECTTO 	   myurl()
#define MANREDIRECTTO 	   my_man_url()
#define AUTOREDIRECTTO 	   my_auto_url()
#define CONTENTSREDIRECTTO my_contents_url()

// This macro causes the FAT library to wait for everything to complete
// before returning to the caller.  This makes the application MUCH simpler.
#define FAT_BLOCK
#use "fat16.lib"

#memmap xmem

// we're on a 24-hr clock here
#define MAX_TIME 2359

FILE far *fp;

struct event {
       int time;
       int duration;
       int zone;
} ;
struct event eventlist[7][20];   // 7 days in week, 20 sprinkle-events/day

int auto_enabled   = FALSE;
int manual_enabled = FALSE;

int current_time = 0;       // 24-hr format; 1:15pm is 1315
int current_day  = 0;	    // Sunday=0
int current_sec  = 0;

#use "dcrtcp.lib"
#use "http.lib"

#include "../../LIBS/RPC_DEFINES.LIB" // Bob Colwell's WFS defines
#include "../../LIBS/RPC_LCD.LIB"     // Bob Colwell's LCD library
#include "../../LIBS/RPC_NETTIME.LIB" // Bob Colwell's "get time from internet" lib
#include "../../LIBS/RPC_RELAYS.LIB"  // Bob Colwell's relay control routines
#include "../../LIBS/RPC_EVENTS.LIB"  // Bob Colwell's autosprinkle event routines

#ximport "WFS_rev1.zhtml"       index_zhtml
#ximport "contents.zhtml"       contents
#ximport "WFS_nav.html"         wfs_nav
#ximport "manual_frame.zhtml"   man_frm
#ximport "auto_frame.zhtml"     auto_frm
#ximport "zones_frame.html"     zones_frm
#ximport "../pages/1.gif"       ledon_gif
#ximport "../pages/0.gif"       ledoff_gif
#ximport "../pages/one.gif"         one_gif
#ximport "../pages/two.gif"         two_gif
#ximport "../pages/three.gif"       three_gif
#ximport "../pages/four.gif"        four_gif
#ximport "../pages/five.gif"        five_gif
#ximport "../pages/six.gif"         six_gif
#ximport "../pages/seven.gif"       seven_gif
#ximport "../pages/eight.gif"       eight_gif
#ximport "../pages/nine.gif"        nine_gif
#ximport "../pages/ten.gif"         ten_gif
#ximport "../pages/eleven.gif"      eleven_gif
#ximport "../pages/twelve.gif"      twelve_gif
#ximport "../pages/thirteen.gif"    thirteen_gif
#ximport "../pages/fourteen.gif"    fourteen_gif
#ximport "../pages/fifteen.gif"     fifteen_gif
#ximport "../pages/sixteen.gif"     sixteen_gif

/* The default mime type for '/' must be first */
SSPEC_MIMETABLE_START
   // This handler enables the ZHTML parser to be used on ZHTML files...
   SSPEC_MIME_FUNC(".zhtml", "text/html", zhtml_handler),
   SSPEC_MIME( ".html", "text/html"),           // html
   SSPEC_MIME( ".cgi", ""),                     // cgi
   SSPEC_MIME( ".gif", "image/gif")
SSPEC_MIMETABLE_END

char *myurl() {
   static char URL[64];
   char tmpstr[32];
   long ipval;

   ifconfig(IF_DEFAULT, IFG_IPADDR, &ipval, IFS_END);
   sprintf(URL, "http://%s/index.zhtml", inet_ntoa(tmpstr, ipval));
   return URL;
}

char *my_man_url() {
   static char URL[64];
   char tmpstr[32];
   long ipval;

   ifconfig(IF_DEFAULT, IFG_IPADDR, &ipval, IFS_END);
   sprintf(URL, "http://%s/manual_frame.zhtml", inet_ntoa(tmpstr, ipval));
   return URL;
}

char *my_auto_url() {
   static char URL[64];
   char tmpstr[32];
   long ipval;

   ifconfig(IF_DEFAULT, IFG_IPADDR, &ipval, IFS_END);
   sprintf(URL, "http://%s/auto_frame.zhtml", inet_ntoa(tmpstr, ipval));
   return URL;
}

char *my_contents_url() {
   static char URL[64];
   char tmpstr[32];
   long ipval;

   ifconfig(IF_DEFAULT, IFG_IPADDR, &ipval, IFS_END);
   sprintf(URL, "http://%s/contents.zhtml", inet_ntoa(tmpstr, ipval));
   return URL;
}

void clear_all_ledflags()
{
   int i;
   for (i=1; i<=16; i++) ledflag[i] = FALSE;
}

int led1toggle(HttpState* state)   // gotta be a way to have one ledxtoggle()
{                                  // and parameterize it, but I don't know how
   if (ledflag[1] == TRUE) {
       ledflag[1] = FALSE;
   } else {
      clear_all_ledflags();
      if (manual_enabled == TRUE) ledflag[1] = TRUE;
   }
   cgi_redirectto(state,MANREDIRECTTO);
   return 0;
}

int led2toggle(HttpState* state)
{
   if (ledflag[2] == TRUE) {
       ledflag[2] = FALSE;
   } else {
      clear_all_ledflags();
      if (manual_enabled == TRUE) ledflag[2] = TRUE;
   }
   cgi_redirectto(state,MANREDIRECTTO);
   return 0;
}

int led3toggle(HttpState* state)
{
   if (ledflag[3] == TRUE) {
       ledflag[3] = FALSE;
   } else {
      clear_all_ledflags();
      if (manual_enabled == TRUE) ledflag[3] = TRUE;
   }
   cgi_redirectto(state,MANREDIRECTTO);
   return 0;
}

int led4toggle(HttpState* state)
{
   if (ledflag[4] == TRUE) {
       ledflag[4] = FALSE;
   } else {
      clear_all_ledflags();
      if (manual_enabled == TRUE) ledflag[4] = TRUE;
   }
   cgi_redirectto(state,MANREDIRECTTO);
   return 0;
}

int led5toggle(HttpState* state)
{
   if (ledflag[5] == TRUE) {
       ledflag[5] = FALSE;
   } else {
      clear_all_ledflags();
      if (manual_enabled == TRUE) ledflag[5] = TRUE;
   }
   cgi_redirectto(state,MANREDIRECTTO);
   return 0;
}

int led6toggle(HttpState* state)
{
   if (ledflag[6] == TRUE) {
       ledflag[6] = FALSE;
   } else {
      clear_all_ledflags();
      if (manual_enabled == TRUE) ledflag[6] = TRUE;
   }
   cgi_redirectto(state,MANREDIRECTTO);
   return 0;
}

int led7toggle(HttpState* state)
{
   if (ledflag[7] == TRUE) {
       ledflag[7] = FALSE;
   } else {
      clear_all_ledflags();
      if (manual_enabled == TRUE) ledflag[7] = TRUE;
   }
   cgi_redirectto(state,MANREDIRECTTO);
   return 0;
}

int led8toggle(HttpState* state)
{
   if (ledflag[8] == TRUE) {
       ledflag[8] = FALSE;
   } else {
      clear_all_ledflags();
      if (manual_enabled == TRUE) ledflag[8] = TRUE;
   }
   cgi_redirectto(state,MANREDIRECTTO);
   return 0;
}

int led9toggle(HttpState* state)
{
   if (ledflag[9] == TRUE) {
       ledflag[9] = FALSE;
   } else {
      clear_all_ledflags();
      if (manual_enabled == TRUE) ledflag[9] = TRUE;
   }
   cgi_redirectto(state,MANREDIRECTTO);
   return 0;
}

int led10toggle(HttpState* state)
{
   if (ledflag[10] == TRUE) {
       ledflag[10] = FALSE;
   } else {
      clear_all_ledflags();
      if (manual_enabled == TRUE) ledflag[10] = TRUE;
   }
   cgi_redirectto(state,MANREDIRECTTO);
   return 0;
}

int led11toggle(HttpState* state)
{
   if (ledflag[11] == TRUE) {
       ledflag[11] = FALSE;
   } else {
      clear_all_ledflags();
      if (manual_enabled == TRUE) ledflag[11] = TRUE;
   }
   cgi_redirectto(state,MANREDIRECTTO);
   return 0;
}

int led12toggle(HttpState* state)
{
   if (ledflag[12] == TRUE) {
       ledflag[12] = FALSE;
   } else {
      clear_all_ledflags();
      if (manual_enabled == TRUE) ledflag[12] = TRUE;
   }
   cgi_redirectto(state,MANREDIRECTTO);
   return 0;
}
int led13toggle(HttpState* state)
{
   if (ledflag[13] == TRUE) {
       ledflag[13] = FALSE;
   } else {
      clear_all_ledflags();
      if (manual_enabled == TRUE) ledflag[13] = TRUE;
   }
   cgi_redirectto(state,MANREDIRECTTO);
   return 0;
}

int led14toggle(HttpState* state)
{
   if (ledflag[14] == TRUE) {
       ledflag[14] = FALSE;
   } else {
      clear_all_ledflags();
      if (manual_enabled == TRUE) ledflag[14] = TRUE;
   }
   cgi_redirectto(state,MANREDIRECTTO);
   return 0;
}

int led15toggle(HttpState* state)
{
   if (ledflag[15] == TRUE) {
       ledflag[15] = FALSE;
   } else {
      clear_all_ledflags();
      if (manual_enabled == TRUE) ledflag[15] = TRUE;
   }
   cgi_redirectto(state,MANREDIRECTTO);
   return 0;
}

int led16toggle(HttpState* state)
{
   if (ledflag[16] == TRUE) {
       ledflag[16] = FALSE;
   } else {
      clear_all_ledflags();
      if (manual_enabled == TRUE) ledflag[16] = TRUE;
   }
   cgi_redirectto(state,MANREDIRECTTO);
   return 0;
}


/* Associate the #ximported files with the web server */
SSPEC_RESOURCETABLE_START
   SSPEC_RESOURCE_XMEMFILE("/", index_zhtml),
   SSPEC_RESOURCE_XMEMFILE("/index.zhtml", index_zhtml),
   SSPEC_RESOURCE_XMEMFILE("/contents.zhtml", contents),
   SSPEC_RESOURCE_XMEMFILE("/WFS_nav.html", wfs_nav),
   SSPEC_RESOURCE_XMEMFILE("/manual_frame.zhtml", man_frm),
   SSPEC_RESOURCE_XMEMFILE("/auto_frame.zhtml", auto_frm),
   SSPEC_RESOURCE_XMEMFILE("/zones_frame.html", zones_frm),
   SSPEC_RESOURCE_XMEMFILE("/1.gif", ledon_gif),
   SSPEC_RESOURCE_XMEMFILE("/0.gif", ledoff_gif),
   SSPEC_RESOURCE_XMEMFILE("/one.gif",   one_gif),
   SSPEC_RESOURCE_XMEMFILE("/two.gif",   two_gif),
   SSPEC_RESOURCE_XMEMFILE("/three.gif", three_gif),
   SSPEC_RESOURCE_XMEMFILE("/four.gif",  four_gif),
   SSPEC_RESOURCE_XMEMFILE("/five.gif",  five_gif),
   SSPEC_RESOURCE_XMEMFILE("/six.gif",   six_gif),
   SSPEC_RESOURCE_XMEMFILE("/seven.gif", seven_gif),
   SSPEC_RESOURCE_XMEMFILE("/eight.gif", eight_gif),
   SSPEC_RESOURCE_XMEMFILE("/nine.gif",  nine_gif),
   SSPEC_RESOURCE_XMEMFILE("/ten.gif",   ten_gif),
   SSPEC_RESOURCE_XMEMFILE("/eleven.gif", eleven_gif),
   SSPEC_RESOURCE_XMEMFILE("/twelve.gif",  twelve_gif),
   SSPEC_RESOURCE_XMEMFILE("/thirteen.gif", thirteen_gif),
   SSPEC_RESOURCE_XMEMFILE("/fourteen.gif", fourteen_gif),
   SSPEC_RESOURCE_XMEMFILE("/fifteen.gif", fifteen_gif),
   SSPEC_RESOURCE_XMEMFILE("/sixteen.gif", sixteen_gif),

   SSPEC_RESOURCE_FUNCTION("/l1tog.cgi", led1toggle),
   SSPEC_RESOURCE_FUNCTION("/l2tog.cgi", led2toggle),
   SSPEC_RESOURCE_FUNCTION("/l3tog.cgi", led3toggle),
   SSPEC_RESOURCE_FUNCTION("/l4tog.cgi", led4toggle),
   SSPEC_RESOURCE_FUNCTION("/l5tog.cgi", led5toggle),
   SSPEC_RESOURCE_FUNCTION("/l6tog.cgi", led6toggle),
   SSPEC_RESOURCE_FUNCTION("/l7tog.cgi", led7toggle),
   SSPEC_RESOURCE_FUNCTION("/l8tog.cgi", led8toggle),
   SSPEC_RESOURCE_FUNCTION("/l9tog.cgi", led9toggle),
   SSPEC_RESOURCE_FUNCTION("/l10tog.cgi", led10toggle),
   SSPEC_RESOURCE_FUNCTION("/l11tog.cgi", led11toggle),
   SSPEC_RESOURCE_FUNCTION("/l12tog.cgi", led12toggle),
   SSPEC_RESOURCE_FUNCTION("/l13tog.cgi", led13toggle),
   SSPEC_RESOURCE_FUNCTION("/l14tog.cgi", led14toggle),
   SSPEC_RESOURCE_FUNCTION("/l15tog.cgi", led15toggle),
   SSPEC_RESOURCE_FUNCTION("/l16tog.cgi", led16toggle)
SSPEC_RESOURCETABLE_END

/*
 * If any zones are on, only show clock info in top LCD line. If
 * no zones are on, show same clock info on top LCD line,
 * day-of-week, day-of-month, and year in bottom LCD line. Top
 * part of this routine is Rabbit's code, rest added by rpc.
 */
void lcd_show_netclk(ntpt, zone_on)
	NTP_ts ntpt;
	unsigned int zone_on;
{
   static char * const weekdays[7] =
		{ "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
   static char * const months[12] =
		{ "Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul",\
       			        "Aug", "Sep", "Oct", "Nov", "Dec" };
   static struct tm daytime;
   longword stamp, frac, g, f, offset;
   int local;
   int dst_tz_correction;   // daylight savings time time-zone correction
   char s[8];

   offset = 0;
   local = 1;
   stamp = ntpt.sec;
   frac = ntpt.frac;

	// g,f is sign extended offset.
   g = offset < 0 ? 0xFFC00000uL : 0;
   f = offset << 22;
   g |= (offset >> 10) & 0x3FFFFFuL;

   if (frac + f < frac)            // Carry in addition
		stamp += g + 1;
      else
		stamp += g;
   frac += f;
   stamp -= EPOCH_DIFF;

   dst_tz_correction = TIMEZONE + dst(daytime.tm_year+1900,
                                      daytime.tm_mon,
                                      daytime.tm_mday);
   if (local)
		mktm(&daytime, stamp + dst_tz_correction*3600L);
	else
		mktm(&daytime, stamp);

   // LCD display looks like this when no zones are active:
   //
   //     12:32:41
   // Mon, Dec. 7 2009

   // month / day / year
   lcd_wr_str(1,0,"    ");
   utoa(daytime.tm_hour, &s[0]);
   if (strlen(s) == 1) {s[2]=s[1]; s[1]=s[0]; s[0]='0';}
   lcd_wr_str(1,4, s);
   lcd_wr_str(1,6, ":");

   utoa(daytime.tm_min, &s[0]);
   if (strlen(s) == 1) {s[2]=s[1]; s[1]=s[0]; s[0]='0';}
   lcd_wr_str(1,7, s);
   lcd_wr_str(1,9, ":");

   utoa(daytime.tm_sec, &s[0]);
   if (strlen(s) == 1) {s[2]=s[1]; s[1]=s[0]; s[0]='0';}
   lcd_wr_str(1,10, s);
   lcd_wr_str(1,12, "    ");

   current_time = (daytime.tm_hour * 100) + daytime.tm_min;  // hr/min -> 24hr format
   current_day  = daytime.tm_wday;
   current_sec  = daytime.tm_sec;

   if (zone_on == SHOW_DATE) {
      lcd_wr_str(2,0, weekdays[daytime.tm_wday]);
      lcd_wr_str(2,3, ", ");

      lcd_wr_str(2,5, months[daytime.tm_mon - 1]);
      lcd_wr_str(2,8, " ");

      utoa(daytime.tm_mday, &s[0]);
      lcd_wr_str(2,9, s);

 	 // gotta catch single-digit day-of-month cases
      if (strlen(s) == 1) {
          lcd_wr_str(2,10," ");
      }
      lcd_wr_str(2,11, " ");

      utoa(daytime.tm_year+1900, &s[0]);
      lcd_wr_str(2,12, s);
   }
}


int dayofweek;
int timeofday;
int duration;
int zone;

/* These #web's register the C variables with the server code so
 * that browsers can read and write them.
 */
#web eventlist[@][@]
#web ledflag

#web manual_enabled select("Disabled" = FALSE, "Enabled" = TRUE)
#web auto_enabled   select("Disabled" = FALSE, "Enabled" = TRUE)
#web sprinkling
#web code_rev
#web code_subrev

#web dayofweek select("Sunday" = 0, "Monday", "Tuesday", "Wednesday",\
						"Thursday", "Friday", "Saturday")

#web timeofday (($timeofday >= 0) && ($timeofday <=2400))
#web duration  (($duration > 0) && ($duration <= 30))

#web zone select("DELETE" = 0, "Zone 1", "Zone 2", "Zone 3", "Zone 4", "Zone 5",\
                 "Zone 6", "Zone 7", "Zone 8", "Zone 9", "Zone 10", "Zone 11",\
                 "Zone 12", "Zone 13", "Zone 14", "Zone 15", "Zone 16")


int event_update(HttpState* state);

#web_update dayofweek, timeofday, duration, zone event_update

// This routine only gets called if one or more of the web vars (dayofweek
// etc) are modified and "submitted" from the client
int event_update()
{
   update_eventlist(dayofweek, timeofday, duration, zone);
   dump_eventlist();
}


// match if current time is >= eventlist[day][any].time and < time+duration
// but don't let times span days, so if current time is 2359, return no match
int current_timedate_match()
{
   int i;

   if (current_time >= MAX_TIME) return NONE;

   for (i=0; i<=19; i++) {
      if ((current_time >= eventlist[current_day][i].time) &&
          (current_time < (eventlist[current_day][i].time +
	                        eventlist[current_day][i].duration)) &&
          (eventlist[current_day][i].zone != 0)) {
               return i;
      }
   }
   return NONE;
}

// This routine handles the manual and automatic zone switching.
// If both manual and auto would be active, manual wins.
void update_outputs(unsigned int *current_zone,
                    unsigned int *next_zone)
{
   struct tm  thetm;
   unsigned long thetime;
   int i, eventlist_index, b2b;
   static int auto_sprinkling = FALSE;

   *next_zone = 0;
   for (i=1; i<=16; i++) {
    	if (ledflag[i] == TRUE) *next_zone = i;
   }

   if ( *next_zone == 0 ) {  // no manual zones on, check auto schedule
        *current_zone = 0;
        eventlist_index = current_timedate_match();

	     if ( ((auto_sprinkling==0) && (eventlist_index!=NONE) && auto_enabled)) {
            if (OPMODE & 0x08)
                printf("DEBUG: auto turning on zone %d.\n",
                                  eventlist[current_day][eventlist_index].zone);
		      //  automode turns on a zone

            relay_disable();
            relay_sel(eventlist[current_day][eventlist_index].zone);
            relay_enable();

	         auto_sprinkling = eventlist[current_day][eventlist_index].zone;
            sprinkling =  auto_sprinkling;
         	lcd_show_netclk(get_local(), SHOW_ZONE);
            lcd_show_zone(eventlist[current_day][eventlist_index].zone, AUTO);
	         return;
        }

        if (auto_sprinkling && (eventlist_index!=NONE) && auto_enabled) {
            if (auto_sprinkling != eventlist[current_day][eventlist_index].zone) {
            //  automode zone A going off, zone B going on
                if (OPMODE & 0x08)
                    printf("DEBUG: auto switching to zone %d.\n",
             	           eventlist[current_day][eventlist_index].zone);
                relay_disable();
                relay_sel(eventlist[current_day][eventlist_index].zone);
                relay_enable();
	             auto_sprinkling = eventlist[current_day][eventlist_index].zone;
                sprinkling = auto_sprinkling;
                lcd_show_netclk(get_local(), SHOW_ZONE);
                lcd_show_zone(eventlist[current_day][eventlist_index].zone, AUTO);
            }

        	   lcd_show_netclk(get_local(), SHOW_ZONE);
            return;
        }

	    if ((auto_sprinkling && (eventlist_index==NONE)) || !auto_enabled) {
            if (auto_enabled && (OPMODE&0x08))
            	printf("DEBUG: auto turning off zones.\n");
		      // automode turns off all zones

            relay_disable();
	         auto_sprinkling = FALSE;
            sprinkling = 0;

            lcd_show_netclk(get_local(), SHOW_DATE);
            if (lcd_beyond_initial_startup == TRUE)
                 BitWrPortI(PDDR, &PDDRShadow, OFF, LCDBacklightEnableBit);
            sprinkling = 0;
            return;
        } else {
            //if (OPMODE & 0x08) printf("DEBUG: auto cont. zone %d.\n",sprinkling);

		      // automode continue sprinkling zone A
            // or not sprinkling any auto zones, as case may be

            if (eventlist_index!=NONE) {
              lcd_show_netclk(get_local(), SHOW_ZONE);
              lcd_show_zone(eventlist[current_day][eventlist_index].zone, AUTO);
            } else {
              lcd_show_netclk(get_local(), SHOW_DATE);
            }

            return;
        }
   }
   else
   {                              // a manual zone is on
        no_lcd_events = FALSE;
        if ((*next_zone == *current_zone) && (manual_enabled)) {
            //don't need to do anything, just keep on sprinkling, update clk
            lcd_show_netclk(get_local(), SHOW_ZONE);
            BitWrPortI(PDDR, &PDDRShadow, ON, LCDBacklightEnableBit);
        } else {
            if (manual_enabled) {
               //Got a new zone to sprinkle. Change port selector to new zone,
               //update LCD display to reflect new zone, enable relays

               auto_enabled = FALSE;  //manual overrides auto
               relay_disable();
               lcd_show_zone(*next_zone, MANUAL);
               relay_sel(*next_zone);   // relay_sel will decrement a
               relay_enable();
               sprinkling = *next_zone;
               *current_zone = *next_zone;
	            lcd_show_netclk(get_local(), SHOW_ZONE);
               BitWrPortI(PDDR, &PDDRShadow, ON, LCDBacklightEnableBit);
            }
        }
   }
}


int rc;
long prealloc;
fat_part *first_part;

int main(void)
{
   // net time vars
   struct tm  thetm;
   unsigned long thetime, t0, t1;

   unsigned current_zone, next_zone;
   int i;

   rc = fat_AutoMount(FDDF_USE_DEFAULT);
   first_part = NULL;
 	for (i = 0; i < num_fat_devices * FAT_MAX_PARTITIONS; ++i) {
		if ((first_part = fat_part_mounted[i]) != NULL) {
			// found a mounted partition, so use it
			break;
		}
	}

	// Check if a mounted partition was found
	if (first_part == NULL) {
		// No mounted partition found, ensure rc is set to a FAT error code.
		rc = (rc < 0) ? rc : -ENOPART;
	} else {
		// It is possible that a non-fatal error was encountered and reported,
		// even though fat_AutoMount() succeeded in mounting at least one
		// FAT partition.
      if (OPMODE & 0x08)
             printf("fat_AutoMount() succeeded with return code %d.\n", rc);
		// We found a partition to work with, so ignore other error (if any).
		rc = 0;
	}
   // FAT return codes always follow the convention that a negative value
   // indicates an error.
	if (rc < 0) {
      // An error occurred.  Here, we print out the numeric code.  You can
      // look in lib\filesystem\errno.lib to see what type of error it
      // really is.  Note that the values in errno.lib are the absolute
      // value of the return code.
   	if (rc == -EUNFORMAT)
      	printf("Device not Formatted, Please run Fmt_Device.c\n");
      else
	   	printf("fat_AutoMount() failed with return code %d.\n", rc);
      exit(1);
   }

   // OK, filesystem exists and is ready to access

   rpc_init();        // in RPC_RELAYS.LIB: init ports A & D to talk to LCD
   		      // and relays
   relay_disable();
   lcd_init();

   lcd_wr_str(2,0,"  Initializing  ");
   lcd_wr_str(1,0," WiFi Sprinkler ");
   lcd_beyond_initial_startup = FALSE;

   clear_all_ledflags();

   code_rev = CODE_REV;           //code revision to show on URL
   code_subrev = CODE_SUBREV;

   current_zone = next_zone = 0;  // zero zone means no sprinklers running
   sprinkling = 0;

	// Initialize the TCP/IP stack and HTTP server
	// Start network and wait for interface to come up (or error exit).
   sock_init_or_exit(1);
   http_init();

	// This yields a performance improvement for an HTTP server
   tcp_reserveport(80);

   init_eventlist();
   boot_file_setup();

   // Find the time from the internet resources.
   get_the_time();
   tm_rd(&thetm);
   t0 = mktime(&thetm);

   while (1) {
      costate {
      	 waitfor (DelayMs(400));
          update_outputs(&current_zone, &next_zone);
      }

      http_handler();  // costates give http handler lion's share of cycles

      costate {
          waitfor (DelayMs(500));
          tm_rd(&thetm);
          t1 = mktime(&thetm);

          // LCD backlight was set on during pwr up init. Turn back off after a few
          // seconds if no zones have been enabled in the meantime; else leave alone.
          if (  (t1 > (t0+LCD_PWRUP_BKLIGHT_TIMEOUT))
             && (lcd_beyond_initial_startup == FALSE))  {
               lcd_beyond_initial_startup = TRUE;
               BitWrPortI(PDDR, &PDDRShadow, OFF, LCDBacklightEnableBit);
          }
      }
   }
}

